/*
 * Copyright (c) 2003, 2004 Henri Sivonen
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package fi.iki.hsivonen.template;
import java.io.IOException;

/**
 * A <code>TemplateEngine</code> gets the templates and document data from 
 * the <code>DataSource</code> that is passed to the constructor of the 
 * <code>TemplateEngine</code>. A class that wishes to provide data to an 
 * instance of <code>TemplateEngine</code> should implement this interface.
 *
 * @version $Id: DataSource.java,v 1.21 2004/08/31 13:45:22 hsivonen Exp $
 * @author  hsivonen
 */
public interface DataSource {

    /**
     * Instantiates a new object that implements the <code>Dependencies</code>
     * interface.
     *
     * @return a new object that implements <code>Dependencies</code> or 
     *         <code>null</null> if dependency tracking not implemented
     */
    public Dependencies newDependencies();

    /**
     * Returns <code>true</code> if the <code>TemplateEngine</code> should 
     * avoid requesting the full content and <code>false</code> if it doesn't
     * matter.
     *
     * @return boolean indicating whether requesting full content should be 
     * avoided.
     */
    public boolean shouldAvoidFullContent();

    /**
     * Returns a <code>List</code> of <code>Item</code>s that is
     * the result of the <code>query</code>
     *
     * @param query an XML element that represents a query for 
     *              selecting items from the data source. The implementing 
     *              class must treat this element as read only.
     * @param context the URL context of the query--that is, the resource being
     *                built in response to a request from a browser
     * @param fullContent   indicates whether the returned 
     *                      <code>Item</code>s should contain
     *                      the full document content and actually be instances
     *                      of either <code>DocumentItem</code> or
     *                      <code>ByteBufferItem</code>
     * @param dependencies the <code>Dependencies</code> instance acquired via
     *                     <code>newDependencies()</code> is passed back so 
     *                     that the dependencies introduced by this query can
     *                     be registered             
     * @return a <code>List</code> of <code>Item</code>s
     *
     * @throws IOException if an I/O error occurs
     * @throws DataSourceException if any other difficulty is encountered
     */
    public java.util.List select(
        org.w3c.dom.Element query,
        java.net.URL context,
        boolean fullContent,
        Dependencies dependencies)
        throws IOException, DataSourceException;

    /**
     * Returns a single item that is the main content for the 
     * <code>context</code> URL or <code>null</code> if the URL doesn't 
     * correspond to a known resource. If non-<code>null</code>, the return 
     * value should either be an instance of <code>DocumentItem</code> or 
     * <code>ByteBufferItem</code>. For example, if the URL points to a 
     * zip file, this method should return the <code>ByteBufferItem</code> 
     * for that zip file. If the main content of the page pointed by the URL is 
     * a single news item, this method should return the 
     * <code>DocumentItem</code> for that news item.
     *
     * @param context the URL context of the query--that is, the resource being
     *                built in response to a request from a browser
     * @param dependencies the <code>Dependencies</code> instance acquired via
     *                     <code>newDependencies()</code> is passed back so 
     *                     that the dependencies introduced by this query can
     *                     be registered  
     * @return the main content for the <code>context</code> URL
     *
     * @throws IOException if an I/O error occurs
     * @throws DataSourceException if any other difficulty is encountered
     */
    public Item select(java.net.URL context, Dependencies dependencies)
        throws DataSourceException, IOException;

    /**
     * Returns the template to be used for building a resource 
     *
     * @param context the URL context of the query--that is, the resource being
     *                built in response to a request from a browser
     * @param dependencies the <code>Dependencies</code> instance acquired via
     *                     <code>newDependencies()</code> is passed back so 
     *                     that the dependencies introduced by this query can
     *                     be registered  
     * @return a Template or <code>null</code> to indicate
     *         templateless pass-though (required for non-XML data)
     *
     * @throws IOException if an I/O error occurs
     * @throws DataSourceException if any other difficulty is encountered
     */
    public Template getTemplateForURL(
        java.net.URL context,
        Dependencies dependencies)
        throws DataSourceException, IOException;
}
