/*
 * Copyright (c) 2003, 2004 Jussi Koskela
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package fi.iki.hsivonen.webcache.dependency;

import java.net.URL;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;

public class CoordinatorThread extends Thread {

    /** Set of queued invalidation keys */
    private LinkedHashSet invalidationSet;

    /** Boolean value indicating if the thread should stop running */
    private volatile boolean stop;

    /** Mapping from cache server URLs to corresponding invalidator threads */
    private LinkedHashMap invalidatorMap;

    /** 
     * Class constructor.
     */
    public CoordinatorThread() {
        invalidationSet = new LinkedHashSet();
        invalidatorMap = new LinkedHashMap();
        stop = false;
    }

    /**
     * Invalidates the content which depends on the given key.
     * @param key the invalidation key.
     */    
    public void invalidate(String key) {
        synchronized (invalidationSet) {
            invalidationSet.add(key);
            invalidationSet.notifyAll();
        }
    }

    /**
     * Starts sending invalidations to the given cache server.
     * @param server the URL of the cache server
     */
    public void startServer(URL server) {
        InvalidatorThread invalidator = null;
        synchronized (invalidatorMap) {
            if (!invalidatorMap.containsKey(server)) {
                invalidator = new InvalidatorThread(server);
                invalidatorMap.put(server, invalidator);
            }
        }
        if (invalidator != null) {
            invalidator.setDaemon(true);
            invalidator.start();
        }
    }

    /**
     * Stops sending invalidations to the given cache server.
     * @param server the URL of the cache server
     */
    public void stopServer(URL server) {
        InvalidatorThread invalidator;
        synchronized (invalidatorMap) {
            invalidator = (InvalidatorThread)invalidatorMap.remove(server);
        }
        if (invalidator != null) {
            invalidator.kill();
        }
    }

    /**
     * Kills the coordinator thread cleanly.
     */
    public void kill() {
        CacheInvalidator.debug("Killing coordinator thread");

        synchronized (invalidationSet) {
            synchronized (invalidatorMap) {
                Iterator iterator = invalidatorMap.values().iterator();
                while (iterator.hasNext()) {
                    InvalidatorThread invalidator = 
                        (InvalidatorThread)iterator.next();
                    invalidator.kill();
                }
            }
                        
            stop = true;
            interrupt();
        }
    }

    /**
     * Starts the coordinator thread.
     */
    public void run() {
        CacheInvalidator.debug("Coordinator thread started");

        stop = false;

        while (!stop) {
            List workList = new LinkedList();

            synchronized (invalidationSet) {
                try {
                    if (invalidationSet.isEmpty()) {
                        invalidationSet.wait();
                    }
                } catch (InterruptedException e) {
                    // somebody wants to kill this thread, let it be so
                }
                workList.addAll(invalidationSet);
                invalidationSet.clear();
            }
            synchronized (invalidatorMap) {
                Iterator iterator = invalidatorMap.values().iterator();
                while (iterator.hasNext()) {
                    InvalidatorThread invalidator = 
                        (InvalidatorThread) iterator.next();
                    invalidator.invalidate(workList);
                }
            }
        }
        
        CacheInvalidator.debug("Coordinator thread stopped");
    }
}
