/*
 * Copyright (c) 2003, 2004 Jussi Koskela
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package fi.iki.hsivonen.webcache.dependency;


/**
 * DependencyTracker keeps track of the dependencies between the
 * database content and the generated http-content.
 */

public class DBSCacheInvalidator extends CacheInvalidator
{
    /** The instance of DBSCacheInvalidator */
    private static DBSCacheInvalidator theInstance;

    /**
     * Returns the instance of DBSCacheInvalidator.
     * @return the instance of DBSCacheInvalidator
     */
    public static synchronized DBSCacheInvalidator getDBSInstance() {
        if (theInstance == null) {
            theInstance = new DBSCacheInvalidator();
        }
        return theInstance;
    }

    /**
     * Invalidates the content which depends on the given domain.
     * @param domain the id of the domain
     */
    public void invalidateDomain(String domain) {
        invalidate(DBSKey.createDomainKey(domain));
    }

    /**
     * Invalidates the content which depends on the given application instance.
     * @param domain the id of the domain
     * @param app the id of the application instance
     */
    public void invalidateAppInstance(String domain, String app) {
        invalidate(DBSKey.createAppInstanceKey(domain, app));
        invalidate(DBSKey.createAppInstanceSetKey(domain));
    }

    /**
     * Invalidates the content which depends on the given file.
     * @param domain the id of the domain, or 
     * null if the file is not domain specific
     * @param file the id of the file
     */	
    public void invalidateFile(String domain, String file) {
        CacheInvalidator.debug("invalidate file : " + domain + " " + file);
        invalidate(DBSKey.createFileKey(domain, file));
    }
 
    /**
     * Invalidates the content which depends on the given item.
     * @param domain the id of the domain
     * @param app the id of the application instance
     * @param item the id of the item
     */	
    public void invalidateItem(String domain, String app, String item) {
        invalidate(DBSKey.createItemKey(domain, app, item));
        invalidate(DBSKey.createItemSetKey(domain, app));
    }

    /**
     * Invalidates the content which depends on the given item set.
     * @param domain the id of the domain
     * @param app the id of the application instance
     */	
    public void invalidateItemSet(String domain, String app) {
        invalidate(DBSKey.createItemSetKey(domain, app));
    }
}
